<?php

namespace Drupal\universal_entity_api\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\universal_entity_api\Service\EntitySerializer;

class BlockListController extends ControllerBase {

  protected $entityTypeManager;
  protected $serializer;

  public function __construct(EntityTypeManagerInterface $manager, EntitySerializer $serializer) {
    $this->entityTypeManager = $manager;
    $this->serializer = $serializer;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('universal_entity_api.serializer')
    );
  }

  public function list($bundle, Request $request) {
    $query = $this->entityTypeManager->getStorage('block_content')->getQuery()
      ->accessCheck(TRUE)
      ->condition('type', $bundle);

    $bids = $query->execute();
    $blocks = $this->entityTypeManager->getStorage('block_content')->loadMultiple($bids);
    $data = array_map(fn($block) => $this->serializer->normalize($block), $blocks);

    return new JsonResponse([
      'count' => count($data),
      'results' => $data,
    ]);
  }
}
