<?php

namespace Drupal\universal_entity_api\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\universal_entity_api\Service\EntitySerializer;
use Symfony\Component\HttpFoundation\RequestStack;

class EntityApiController extends ControllerBase {

  protected $entityTypeManager;
  protected $serializer;
  protected $requestStack;

  public function __construct(
    EntityTypeManagerInterface $manager,
    EntitySerializer $serializer,
    RequestStack $request_stack
  ) {
    $this->entityTypeManager = $manager;
    $this->serializer = $serializer;
    $this->requestStack = $request_stack;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('universal_entity_api.serializer'),
      $container->get('request_stack')
    );
  }

  public function handle($entity_type, $bundle, $id) {
    $entity = $this->entityTypeManager->getStorage($entity_type)->load($id);
    if (!$entity || $entity->bundle() !== $bundle) {
      return new JsonResponse(['error' => 'Entity not found or bundle mismatch.'], 404);
    }

    $request = $this->requestStack->getCurrentRequest();
    $fields = $request->query->get('fields');

    $data = $this->serializer->normalize($entity, ['fields' => $fields]);
    $response = new JsonResponse($data);
    $response->setMaxAge(3600);
    $response->headers->set('Cache-Control', 'public');

    return $response;
  }
}
