<?php

namespace Drupal\universal_entity_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

class EntityTypeListController extends ControllerBase {

  protected $entityTypeManager;

  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  public function list() {
    $entity_types = $this->entityTypeManager->getDefinitions();
    $output = [];

    foreach ($entity_types as $type_id => $definition) {
      $bundles = [];
      if ($bundle_entity_type = $definition->getBundleEntityType()) {
        $bundle_storage = $this->entityTypeManager->getStorage($bundle_entity_type);
        $bundle_entities = $bundle_storage->loadMultiple();
        foreach ($bundle_entities as $bundle) {
          $bundles[] = $bundle->label();
        }
      }
      $output[$type_id] = [
        'label' => $definition->getLabel(),
        'bundles' => $bundles,
      ];
    }

    return new JsonResponse($output);
  }
}
