<?php

namespace Drupal\universal_entity_api\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Controller\ControllerBase;
use Drupal\universal_entity_api\Service\LayoutBuilderNormalizer;

class LayoutApiController extends ControllerBase {

  protected $normalizer;

  public function __construct(LayoutBuilderNormalizer $normalizer) {
    $this->normalizer = $normalizer;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('universal_entity_api.layout_builder_normalizer')
    );
  }

  public function layout($entity_type, $entity_id) {
    $storage = \Drupal::entityTypeManager()->getStorage($entity_type);
    $entity = $storage->load($entity_id);

    if (!$entity) {
      throw new NotFoundHttpException('Entity not found');
    }

    $layout = $this->normalizer->getLayoutStructure($entity);

    return new JsonResponse([
      'entity_type' => $entity_type,
      'entity_id' => $entity_id,
      'layout' => $layout,
      'changed' => $entity->getChangedTime() ?? NULL,
    ]);
  }
}
