<?php

namespace Drupal\universal_entity_api\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

class MenuController extends ControllerBase {

  protected $menuTree;

  public function __construct(MenuLinkTreeInterface $menuTree) {
    $this->menuTree = $menuTree;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('menu.link_tree')
    );
  }

 public function getMenu($menu_id, Request $request) {
  $depth = max(0, (int) $request->query->get('depth', 0));

  $params = new \Drupal\Core\Menu\MenuTreeParameters();
  if ($depth > 0) {
    $params->setMaxDepth($depth);
  }

  $tree = $this->menuTree->load($menu_id, $params);

  $buildTree = function ($items) use (&$buildTree) {
    $output = [];
    foreach ($items as $item) {
      $link = $item->link;
      $url = $link->getUrlObject();
      $output[] = [
        'title' => $link->getTitle(),
        'url' => $url->toString(),
        'enabled' => $link->isEnabled(),
        'description' => $link->getDescription(),
        'weight' => $item->link->getWeight(),
        'external' => $url->isExternal(),
        'children' => $buildTree($item->subtree),
        ];
    }
    return $output;
  };

  return new JsonResponse($buildTree($tree));
}

}
