<?php

namespace Drupal\universal_entity_api\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\universal_entity_api\Service\EntitySerializer;

class NodeListController extends ControllerBase {

  protected $entityTypeManager;
  protected $serializer;

  public function __construct(EntityTypeManagerInterface $manager, EntitySerializer $serializer) {
    $this->entityTypeManager = $manager;
    $this->serializer = $serializer;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('universal_entity_api.serializer')
    );
  }

  public function list($bundle, Request $request) {
    $page = max(0, (int) $request->query->get('page', 0));
    $limit = min(100, (int) $request->query->get('limit', 10));
    $offset = $page * $limit;
    $sort = $request->query->get('sort');
    $fields = $request->query->get('fields');
    $includeFields = $fields ? explode(',', $fields) : [];

    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->accessCheck(TRUE)
      ->condition('type', $bundle)
      ->condition('status', 1)
      ->range($offset, $limit);

    if ($sort) {
      $direction = 'ASC';
      if (str_starts_with($sort, '-')) {
        $direction = 'DESC';
        $sort = substr($sort, 1);
      }
      $query->sort($sort, $direction);
    }

    foreach ($request->query->all() as $key => $value) {
      if (in_array($key, ['page', 'limit', 'sort', 'fields'])) continue;
      if (is_array($value)) {
        $query->condition($key, $value, 'IN');
      } else {
        $query->condition($key, $value);
      }
    }

    $nids = $query->execute();
    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);
    $fields = $request->query->get('fields');
    $data = array_map(fn($node) => $this->serializer->normalize($node, ['fields' => $fields]), $nodes);

    return new JsonResponse([
      'page' => $page,
      'limit' => $limit,
      'count' => count($data),
      'results' => $data,
    ]);
  }
}
