<?php

namespace Drupal\universal_entity_api\Normalizer;

use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\media\MediaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class MediaNormalizer {

  protected $fileUrlGenerator;

  public function __construct(FileUrlGeneratorInterface $fileUrlGenerator) {
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_url_generator')
    );
  }

  public function normalize(MediaInterface $media) {
    if (!$media->hasField('field_media_image')) {
      return [];
    }

    $image_field = $media->get('field_media_image');
    if ($image_field->isEmpty()) {
      return [];
    }

    $file = $image_field->entity;
    if (!$file) {
      return [];
    }

    return [
      'url' => $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri()),
      'alt' => $image_field->alt ?? '',
      'title' => $image_field->title ?? '',
      'mime' => $file->getMimeType(),
    ];
  }
}
