<?php

namespace Drupal\universal_entity_api\Service;

use Drupal\Core\Entity\EntityInterface;
use Drupal\universal_entity_api\Normalizer\FieldNormalizer;

class EntitySerializer {

  protected $fieldNormalizer;

  public function __construct(FieldNormalizer $fieldNormalizer) {
    $this->fieldNormalizer = $fieldNormalizer;
  }

  public function normalize(EntityInterface $entity, array $context = []) {
    $fields = $entity->getFields();
    $field_filter = [];

    // Support for both ?fields=title,body and ?fields[]=title&fields[]=body
    if (!empty($context['fields'])) {
      if (is_string($context['fields'])) {
        $requested = array_map('trim', explode(',', $context['fields']));
      } elseif (is_array($context['fields'])) {
        $requested = array_map('trim', $context['fields']);
      } else {
        $requested = [];
      }
      $field_filter = array_flip($requested);
    }

    $data = [
      'id' => $entity->id(),
      'type' => [
        'id' => $entity->bundle(),
        'type' => $entity->getEntityTypeId() . '_type',
        'bundle' => $entity->getEntityTypeId() . '_type',
        'label' => $entity->getEntityType()->getLabel(),
      ],
      'bundle' => $entity->bundle(),
      'label' => $entity->label(),
    ];

    foreach ($fields as $field_name => $field) {
      if (!empty($field_filter) && !isset($field_filter[$field_name])) {
        continue; // Skip fields not requested
      }
      $data[$field_name] = $this->fieldNormalizer->normalize($field);
    }

    return $data;
  }

}
